//Library headers. Necessary if you are using Reef Angel Controller
#include <Wire.h>
#include <EEPROM.h>
#include <OneWire.h>
#include <Time.h>
#include <DS1307RTC.h>
#include <NokiaLCD.h>
#include <ReefAngel.h>

/* 
----------------------------------------------------------------------------------------------------------------------------------------------------------
Give labels to your outlets
----------------------------------------------------------------------------------------------------------------------------------------------------------
*/ 
#define Sump                8
#define Heater              7
#define Chiller             6
#define WM1                 5
#define WM2                 4
#define MHlight             3
#define Actinic             2
#define AutoTopOff          1


/* 
----------------------------------------------------------------------------------------------------------------------------------------------------------
Create your menu
Make sure the string is no longer than 20 characters.
----------------------------------------------------------------------------------------------------------------------------------------------------------
*/ 
#include <avr/pgmspace.h>
prog_char menu0_label[] PROGMEM = "Feeding Mode";
prog_char menu1_label[] PROGMEM = "Water Change Mode";
prog_char menu2_label[] PROGMEM = "Override Lights On";
prog_char menu3_label[] PROGMEM = "Cancel Override On";
prog_char menu4_label[] PROGMEM = "Clear ATO Timeout";
prog_char menu5_label[] PROGMEM = "Clear Overheat";
PROGMEM const char *menu_items[] = {menu0_label, menu1_label, menu2_label, menu3_label, menu4_label,menu5_label};


/* 
----------------------------------------------------------------------------------------------------------------------------------------------------------
This block runs only once at start-up. It's called setup() function.
It initializes the controller, builds the menu system and turns on the pumps/powerheads
A couple of assumptions were made in this code:
Temp1 is water temperature sensor readings
Temp2 is lights temperature sensor readings
----------------------------------------------------------------------------------------------------------------------------------------------------------
*/ 
void setup()
{
  ReefAngel.Init();  //Initialize controller
  ReefAngel.LoadMenu(pgm_read_word(&(menu_items[0])),SIZE(menu_items));  // Load Menu items from above
  ReefAngel.Relay.On(Sump);  //Turn Sump on at startup
  ReefAngel.Relay.On(WM1);  //Turn WM1 on at startup
  ReefAngel.Relay.On(WM2);  //Turn WM2 on at startup
  //ReefAngel.Timer[0].Interval=10;  // Uncomment to override feeding timer. Default is 900 seconds
  //ReefAngel.Timer[3].Interval=5;  // Uncomment to override screen saver timer. Default is 600 seconds
  //ReefAngel.Timer[3].Start();   // Uncomment to start screen saver timer if you uncommented line above
}

/* 
----------------------------------------------------------------------------------------------------------------------------------------------------------
This block runs forever, over and over again. It's called loop() function.
----------------------------------------------------------------------------------------------------------------------------------------------------------
*/ 
void loop()
{
  ReefAngel.StandardGUI();  //Draw the Standard GUI

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  This block is where you setup your main functions
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  ReefAngel.MHLights(MHlight,15,30,21,00,30);  // Setup Daylight to turn on at 3:30PM and off at 9:00PM  /  Delay 30mins in case of power failure
  ReefAngel.StandardLights(Actinic,15,0,21,30);  // Setup Actinic to turn on at 3:00PM and off at 9:30PM
  ReefAngel.StandardHeater(Heater,789,791);  // Setup Heater to turn on at 78.9F and off at 79.1F
  ReefAngel.StandardFan(Chiller,792,794);  // Setup Chiller to turn on at 79.4F and off at 79.2F
  ReefAngel.StandardATO(AutoTopOff,60);  //Setup ATO to turn on at ATOlow and off at ATOhigh / Timeout set to 60s


  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on default mode and it is showing the default screen, the value of SelectedMenu will be 255.
  So, we check to see if the value is 255 and if it is, it'll check whether or not it should toggle the wavemaker powerheads WM1 and WM2 on or off.
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==255)  // Default Mode - Main Screen
  {
    if (ReefAngel.Timer[1].IsTriggered())  // If timer 1 expires
    {
      ReefAngel.Timer[1].Start();  // start timer
      ReefAngel.Relay.Toggle(WM1);  // toggle relay
    }
    if (ReefAngel.Timer[2].IsTriggered())  // If timer 2 expires
    {
      ReefAngel.Timer[2].Start();  // start timer
      ReefAngel.Relay.Toggle(WM2);  // toggle relay
    }
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on "Feeding Mode", the value of SelectedMenu will be 0. This is based according to the menu created above
  So, we check to see if the value is 0 and if it is, it'll turn Sump, WM1 and WM2 off. It will then display the feeding mode screen
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==0)  // Feeding Mode Selected
  {
    ReefAngel.Relay.Off(Sump);  //Turn Sump off
    ReefAngel.Relay.Off(WM1);  //Turn WM1 off
    ReefAngel.Relay.Off(WM2);  //Turn WM2 off
    ReefAngel.FeedingModeGUI();  // Draw FeedingModeGUI
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on "Water Change Mode", the value of SelectedMenu will be 1. This is based according to the menu created above
  So, we check to see if the value is 1 and if it is, it'll turn Sump, WM1 and WM2 off. It will then display the water change mode screen
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==1)  // Water Change Mode Selected
  {
    ReefAngel.Relay.Off(Sump);  //Turn Sump off
    ReefAngel.Relay.Off(WM1);  //Turn WM1 off
    ReefAngel.Relay.Off(WM2);  //Turn WM2 off
    ReefAngel.WaterChangeModeGUI();  // Draw WaterChangeModeGUI
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on "Override Lights On", the value of SelectedMenu will be 2. This is based according to the menu created above
  So, we check to see if the value is 2 and if it is, it'll set the relay mask to override the outlet 2 and 3 to always on, which are the MHlights and Actinic
  It will also set the pwm channels to 50%, in case you are using LEDs.
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==2)  //Override Lights On Selected
  {
    ReefAngel.Relay.RelayMaskOn=B00000110;  //Override Lights On;
    ReefAngel.PWM.SetActinic(50);  // Set PWM value
    ReefAngel.PWM.SetDaylight(50);  // Set PWM value
    ReefAngel.LCD.DrawText(19,255,10,10,"Override Lights On");
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on "Cancel Override On", the value of SelectedMenu will be 3. This is based according to the menu created above
  So, we check to see if the value is 3 and if it is, it'll set the relay mask to override the outlet 2 and 3 back to auto, which are the MHlights and Actinic
  It will also set the pwm channels to 0%, in case you are using LEDs.
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==3)  //Override Lights Off Selected
  {
    ReefAngel.Relay.RelayMaskOn=B00000000;  //Override Lights Off;
    ReefAngel.PWM.SetActinic(0);  // Set PWM value
    ReefAngel.PWM.SetDaylight(0);  // Set PWM value
    ReefAngel.ReturnMenuFunction();  // Return from menu function;
  }  

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on "Clear ATO Timeout", the value of SelectedMenu will be 4. This is based according to the menu created above
  So, we check to see if the value is 4 and if it is, it'll clear the ATO timeout flag and turn the status LED off
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==4)  //Clear ATO Timeout Selected
  {
    ReefAngel.LED.Off();
    ReefAngel.ATO.topping=false;
    ReefAngel.ReturnMenuFunction();  // Return from menu function;
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If the controller is on "Clear Overheat", the value of SelectedMenu will be 5. This is based according to the menu created above
  So, we check to see if the value is 5 and if it is, it'll clear the Overheat
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.SelectedMenu==5)  //Clear Overheat Selected
  {
    ReefAngel.LED.Off();
    ReefAngel.Relay.RelayMaskOn=B11111111;  //Override MHlight Auto;
    ReefAngel.ReturnMenuFunction();  // Return from menu function;
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  The Timer[0] is used as feeding mode timer.
  When Timer[0] is triggered, it will turn the Sump on.
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.Timer[0].IsTriggered())  // If feeding timer runs out
  {
    ReefAngel.ReturnMenuFunction();  // Return from menu function;
    ReefAngel.Relay.On(Sump);  //Turn Sump off
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  If Temp2 sensor reachs 150.0F, it will consider an overheat and will turn the lights off
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  if (ReefAngel.Params.Temp2>=1500)  // It Temp2 >= 150.0F
  {
    ReefAngel.LED.On();
    ReefAngel.Relay.RelayMaskOn=B11111011;  //Override MHlight Off;
  }

  /* 
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  Writes the updated value to the relay box.
  ----------------------------------------------------------------------------------------------------------------------------------------------------------
  */ 
  ReefAngel.Relay.Write();  // Make relay changes effective
}
